package AccordionBus;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.util.LinkedList;
import java.util.Random;

// Форма для работы с набором объектов класса DrawingBus
public class BusCollectionForm extends JFrame {
    // Набор объектов
    private final BusGenericStorage _storage;
    // Набор удаленных объектов
    private final LinkedList<DrawingBus> _deletedBuses;
    // Выбранный автобус
    public DrawingBus SelectedBus;
    private FrameBusGeneric frameBusGeneric;

    private JPanel mainPanel;
    public JPanel getMainPanel() {
        return mainPanel;
    }
    private JPanel pictureBoxCollection;
    private JButton buttonAddBus;
    private JTextField textFieldNumber;
    private JButton buttonRemoveBus;
    private JButton buttonRefreshCollection;
    private JPanel toolsPanel;
    private JLabel toolsLabel;
    private JButton buttonGenerateBus;
    private JTextField textFieldStorage;
    private JButton buttonAddObject;
    private DefaultListModel listModel;
    private JList listStorages;
    private JButton buttonDeleteObject;
    private JButton buttonShowDeletedBus;

    // Конструктор
    public BusCollectionForm() {
        pictureBoxCollection.setSize(new Dimension(700, 450));
        _storage = new BusGenericStorage(pictureBoxCollection.getWidth(), pictureBoxCollection.getHeight());
        _deletedBuses = new LinkedList<DrawingBus>();
        listModel = new DefaultListModel<String>();
        listStorages.setModel(listModel);

        // Добавление набора
        buttonAddObject.addActionListener(e -> {
            if (textFieldStorage.getText().length() == 0) {
                JOptionPane.showMessageDialog(null, "Не все данные заполнены", "Ошибка", JOptionPane.ERROR_MESSAGE);
                return;
            }
            _storage.AddSet(textFieldStorage.getText());
            ReloadObjects();
        });

        // Выбор набора
        listStorages.addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                Refresh();
            }
        });

        // Удаление набора
        buttonDeleteObject.addActionListener(e -> {
            if (listStorages.getSelectedIndex() == -1) {
                return;
            }

            if (JOptionPane.showConfirmDialog(null, "Удалить объект " + listStorages.getSelectedValue() + "?",  "Удаление", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
                _storage.DelSet(listStorages.getSelectedValue().toString());
                ReloadObjects();
            }
        });

        // Добавление объета в набор
        buttonAddBus.addActionListener(e -> {
            if (listStorages.getSelectedIndex() == -1) {
                return;
            }

            var obj = _storage.get(listStorages.getSelectedValue().toString());
            if (obj == null) {
                return;
            }

            FrameBusConfig frameBusConfig = new FrameBusConfig();
            frameBusConfig.setVisible(true);

            frameBusConfig.busConfigForm.addButton.addActionListener(ev -> {
                SelectedBus = frameBusConfig.busConfigForm._bus;
                frameBusConfig.dispose();
                if (SelectedBus != null) {
                    if (obj.plus(SelectedBus) > -2) {
                        Refresh();
                        JOptionPane.showMessageDialog(this.getMainPanel(), "Объект добавлен", "Успех", JOptionPane.INFORMATION_MESSAGE);
                    }
                    else {
                        JOptionPane.showMessageDialog(this.getMainPanel(), " Объект не добавлен", "Ошибка", JOptionPane.INFORMATION_MESSAGE);
                    }
                }
            });

            frameBusConfig.busConfigForm.cancelButton.addActionListener(ee -> frameBusConfig.dispose());
        });

        // Удаление объекта из набора
        buttonRemoveBus.addActionListener(e -> {
            if (listStorages.getSelectedIndex() == -1) {
                return;
            }

            var obj = _storage.get(listStorages.getSelectedValue().toString());
            if (obj == null) {
                return;
            }

            if (JOptionPane.showConfirmDialog(null, "Удалить объект?", "Удаление", JOptionPane.YES_NO_OPTION) == JOptionPane.NO_OPTION) {
                return;
            }

            int pos = Integer.parseInt(textFieldNumber.getText());
            var removed = obj.minus(pos);
            if (removed != null) {
                _deletedBuses.add(removed);
                JOptionPane.showMessageDialog(null, "Объект удален", "Удаление", JOptionPane.INFORMATION_MESSAGE);
                Refresh();
            }
            else {
                JOptionPane.showMessageDialog(null, "Не удалось удалить объект", "Удаление", JOptionPane.INFORMATION_MESSAGE);
            }
        });

        // Обновить коллекцию
        buttonRefreshCollection.addActionListener(e -> {
            Refresh();
        });

        // Сгенерировать автобус
        buttonGenerateBus.addActionListener(e -> {
            if (frameBusGeneric != null) {
                frameBusGeneric.dispose();
            }

            frameBusGeneric = new FrameBusGeneric();
            frameBusGeneric.setVisible(true);
        });

        // Показать удаленный автобус
        buttonShowDeletedBus.addActionListener(e -> {
            if (_deletedBuses.size() == 0) {
                JOptionPane.showMessageDialog(null, "Удаленные автобусы отсутствуют", "Ошибка", JOptionPane.ERROR_MESSAGE);
                return;
            }

            Random random = new Random();
            FrameAccordionBus frameAccordionBus = new FrameAccordionBus();
            frameAccordionBus.setVisible(true);
            frameAccordionBus.accordionBusForm.drawingBus = _deletedBuses.pop();
            frameAccordionBus.accordionBusForm.drawingBus.SetPosition(random.nextInt(100), random.nextInt(100));
            frameAccordionBus.accordionBusForm.Draw();
            frameAccordionBus.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        });
    }

    // Заполнение ListBoxObject
    private void ReloadObjects() {
        int index = listStorages.getSelectedIndex();

        listModel.clear();
        for (String key : _storage.Keys()) {
            listModel.addElement(key);
        }

        if (listModel.size() > 0 && (index == -1 || index >= listModel.size())) {
            listStorages.setSelectedIndex(0);
        }
        else if (listModel.size() > 0 && index > -1 && index < listModel.size()) {
            listStorages.setSelectedIndex(index);
        }
    }

    // Обновить картинку
    public void Refresh() {
        Graphics g = mainPanel.getGraphics();
        mainPanel.paint(g);

        if (listStorages.getSelectedIndex() == -1) {
            return;
        }

        var obj = _storage.get(listStorages.getSelectedValue().toString());
        if (obj == null) {
            return;
        }

        obj.ShowBuses(g);
    }
}
