package AccordionBus;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.Console;
import java.io.IOException;

public class BusConfigForm {
    // Переменная выбранный автобус
    DrawingBus _bus = null;

    // Переменная выбранный тип дверей
    IDrawingDoors _doors = null;

    // Компоненты DoorComponent
    DoorComponent doorComponentTypeOne = new DoorComponent(new DrawingDoorsTypeOne());
    DoorComponent doorComponentTypeTwo = new DoorComponent(new DrawingDoorsTypeTwo());
    DoorComponent doorComponentTypeThree = new DoorComponent(new DrawingDoorsTypeThree());

    private JPanel configPanel;
    public JPanel getConfigPanel() {
        return configPanel;
    }
    private JPanel objectPanel;
    public JButton addButton;
    public JButton cancelButton;
    private JLabel bodyColorLabel;
    private JLabel additionalColorLabel;
    private JLabel doorsTypeLabel;
    private JPanel pictureBoxPanel;
    private JPanel parametersPanel;
    private JLabel speedSpinnerLabel;
    private JLabel weightSpinnerLabel;
    private JLabel doorsSpinnerLabel;
    private JSpinner speedSpinner;
    private JSpinner weightSpinner;
    private JSpinner doorsSpinner;
    private JCheckBox additionalBodyCheckBox;
    private JCheckBox additionalDoorCheckBox;
    private JPanel colorPanel;
    private JPanel redPanel;
    private JPanel greenPanel;
    private JPanel bluePanel;
    private JPanel yellowPanel;
    private JPanel whitePanel;
    private JPanel grayPanel;
    private JPanel blackPanel;
    private JPanel magentaPanel;
    private JLabel doorsTypeOneLabel = new JLabel("Дверь 1 типа");
    private JLabel doorsTypeTwoLabel = new JLabel("Дверь 2 типа");
    private JLabel doorsTypeThreeLabel = new JLabel("Дверь 3 типа");
    private JLabel simpleObjectLabel;
    private JLabel modifiedObjectLabel;
    private JPanel componentPanel;

    // Конструктор
    public BusConfigForm() {
        // Устанавливаем границы Label
        simpleObjectLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        modifiedObjectLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        doorsTypeOneLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        doorsTypeTwoLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        doorsTypeThreeLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        bodyColorLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        additionalColorLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        doorsTypeLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));

        // Добавляем компоненты DoorComponent на форму
        componentPanel.setLayout(new GridLayout());
        componentPanel.add(doorComponentTypeOne);
        componentPanel.add(doorComponentTypeTwo);
        componentPanel.add(doorComponentTypeThree);

        doorComponentTypeOne.setLayout(new GridLayout(1, 1));
        doorComponentTypeTwo.setLayout(new GridLayout(1, 1));
        doorComponentTypeThree.setLayout(new GridLayout(1, 1));

        doorComponentTypeOne.add(doorsTypeOneLabel);
        doorComponentTypeTwo.add(doorsTypeTwoLabel);
        doorComponentTypeThree.add(doorsTypeThreeLabel);

        // Устанавливаем допустимые значения Spinner
        speedSpinner.setModel(new SpinnerNumberModel(100, 100, 1000, 1));
        weightSpinner.setModel(new SpinnerNumberModel(100, 100, 1000, 1));
        doorsSpinner.setModel(new SpinnerNumberModel(3, 3, 5, 1));

        // Устанавливаем возможность перетаскивания Panel (цвета объекта)
        redPanel.setTransferHandler(new PanelTransferHandler());
        greenPanel.setTransferHandler(new PanelTransferHandler());
        bluePanel.setTransferHandler(new PanelTransferHandler());
        yellowPanel.setTransferHandler(new PanelTransferHandler());
        whitePanel.setTransferHandler(new PanelTransferHandler());
        grayPanel.setTransferHandler(new PanelTransferHandler());
        blackPanel.setTransferHandler(new PanelTransferHandler());
        magentaPanel.setTransferHandler(new PanelTransferHandler());

        redPanel.addMouseListener(new PanelMouseAdapter());
        greenPanel.addMouseListener(new PanelMouseAdapter());
        bluePanel.addMouseListener(new PanelMouseAdapter());
        yellowPanel.addMouseListener(new PanelMouseAdapter());
        whitePanel.addMouseListener(new PanelMouseAdapter());
        grayPanel.addMouseListener(new PanelMouseAdapter());
        blackPanel.addMouseListener(new PanelMouseAdapter());
        magentaPanel.addMouseListener(new PanelMouseAdapter());

        // Устанавливаем возможность перетаскивания Label (типа объекта)
        simpleObjectLabel.setTransferHandler(new LabelTransferHandler());
        modifiedObjectLabel.setTransferHandler(new LabelTransferHandler());

        simpleObjectLabel.addMouseListener(new LabelMouseAdapter());
        modifiedObjectLabel.addMouseListener(new LabelMouseAdapter());

        // Действия при приеме получаемой информации (получение типа объекта)
        pictureBoxPanel.setTransferHandler(new TransferHandler() {
            @Override
            public boolean canImport(TransferHandler.TransferSupport support) {
                return support.isDataFlavorSupported(DataFlavor.stringFlavor);
            }

            @Override
            public boolean importData(TransferHandler.TransferSupport support) {
                if (!canImport(support)) {
                    return false;
                }

                try {
                    String data = (String)support.getTransferable().getTransferData(DataFlavor.stringFlavor);
                    switch (data) {
                        case "Простой":
                            _bus = new DrawingBus((int)speedSpinner.getValue(),
                                    (int) weightSpinner.getValue(),
                                    Color.WHITE,
                                    900,
                                    500);
                            _bus.drawingDoors.setNumDoors((int)doorsSpinner.getValue());
                            break;
                        case "Продвинутый":
                            _bus = new DrawingAccordionBus((int)speedSpinner.getValue(),
                                    (int) weightSpinner.getValue(),
                                    Color.WHITE,
                                    Color.BLACK,
                                    additionalBodyCheckBox.isSelected(),
                                    additionalDoorCheckBox.isSelected(),
                                    900,
                                    500);
                            _bus.drawingDoors.setNumDoors((int)doorsSpinner.getValue());
                            break;
                    }
                } catch (UnsupportedFlavorException | IOException e) {
                    e.printStackTrace();
                }

                DrawBus();
                return true;
            }
        });

        // Действия при приеме получаемой информации (получение основного цвета объекта)
        bodyColorLabel.setTransferHandler(new LabelTransferHandler() {
            @Override
            public boolean canImport(TransferHandler.TransferSupport support) {
                return support.isDataFlavorSupported(ColorTransferable.colorDataFlavor);
            }

            @Override
            public boolean importData(TransferHandler.TransferSupport support) {
                if (!canImport(support)) {
                    return false;
                }

                if (_bus instanceof DrawingBus) {
                    try {
                        Color color = (Color)support.getTransferable().getTransferData(ColorTransferable.colorDataFlavor);
                        bodyColorLabel.setBackground(color);
                        _bus.SetBodyColor(color);
                    }
                    catch (UnsupportedFlavorException | IOException e) {
                        e.printStackTrace();
                    }
                }

                DrawBus();
                return true;
            }
        });

        // Действия при приеме получаемой информации (получение дополнительного цвета объекта)
         additionalColorLabel.setTransferHandler(new LabelTransferHandler() {
            @Override
            public boolean canImport(TransferHandler.TransferSupport support) {
                return support.isDataFlavorSupported(ColorTransferable.colorDataFlavor);
            }

            @Override
            public boolean importData(TransferHandler.TransferSupport support) {
                if (!canImport(support)) {
                    return false;
                }

                if (_bus instanceof DrawingAccordionBus) {
                    try {
                        Color color = (Color)support.getTransferable().getTransferData(ColorTransferable.colorDataFlavor);
                        additionalColorLabel.setBackground(color);
                        ((DrawingAccordionBus)_bus).SetAdditionalColor(color);
                    }
                    catch (UnsupportedFlavorException | IOException e) {
                        e.printStackTrace();
                    }
                }

                DrawBus();
                return true;
            }
        });

        // Действия при приеме получаемой информации (получение типа дверей)
         doorsTypeLabel.setTransferHandler(new TransferHandler() {
             @Override
             public boolean canImport(TransferHandler.TransferSupport support) {
                 return support.isDataFlavorSupported(IDrawingDoorsTransferable.IDrawingDoorsDataFlavor);
             }

             @Override
             public boolean importData(TransferHandler.TransferSupport support) {
                 if (!canImport(support)) {
                     return false;
                 }

                 try {
                     IDrawingDoors drawingDoors = (IDrawingDoors)support.getTransferable().getTransferData(IDrawingDoorsTransferable.IDrawingDoorsDataFlavor);
                     drawingDoors.setNumDoors((int)doorsSpinner.getValue());
                     _bus.drawingDoors = drawingDoors;
                 }
                 catch (UnsupportedFlavorException | IOException e) {
                     e.printStackTrace();
                 }

                 DrawBus();
                 return true;
             }
         });
    }

    // Отрисовать автобус
    private void DrawBus () {
        if (_bus == null) {
            return;
        }

        Graphics g = pictureBoxPanel.getGraphics();
        _bus.SetPosition((pictureBoxPanel.getWidth() - _bus.GetWidth()) / 2, (pictureBoxPanel.getHeight() - _bus.GetHeigth()) / 2);
        pictureBoxPanel.paint(g);
        _bus.DrawTransport(g);
    }
}