package AccordionBus;

import java.awt.*;

// Параметризованный класс для набора объектов DrawingBus
public class BusGenericCollection<T extends DrawingBus, U extends IMoveableObject> {
    // Ширина окна прорисовки
    private final int _pictureWidth;

    // Высота окна прорисовки
    private final int _pictureHeight;

    // Размер занимаемого объектом места (ширина)
    private final int _placeSizeWidth = 215;

    // Размер занимаемого объектом места (высота)
    private final int _placeSizeHeight = 50;

    // Набор объектов
    private final SetGeneric<T> _collection;

    // Конструктор
    public BusGenericCollection(int picWidth, int picHeight) {
        int width = picWidth / _placeSizeWidth;
        int height = picHeight / _placeSizeHeight;
        _pictureWidth = picWidth;
        _pictureHeight = picHeight;

        _collection = new SetGeneric<T>(width * height);
    }

    // Перегрузка оператора сложения
    public int plus(T obj) {
        if (obj == null) {
            return -1;
        }

        return _collection.Insert(obj);
    }

    // Перегрузка оператора вычитания
    public T minus(int pos) {
        T obj = _collection.Get(pos);

        if (obj != null) {
            _collection.Remove(pos);
        }

        return obj;
    }

    // Получение объекта IMoveableObject
    public U GetU(int pos)
    {
        return (U)_collection.Get(pos).GetMoveableObject();
    }

    // Вывод всего набора объектов
    public void ShowBuses(Graphics g) {
        DrawBackground(g);
        DrawObjects(g);
    }

    // Метод отрисовки фона
    private void DrawBackground(Graphics g) {
        Graphics2D g2d = (Graphics2D)g;
        g2d.setColor(Color.BLACK);
        g2d.setStroke(new BasicStroke(3));
        for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++) {
            for (int j = 0; j < _pictureHeight / _placeSizeHeight + 1; j++) {
                // Линия разметки места
                g2d.drawLine(i * _placeSizeWidth, j * _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2, j * _placeSizeHeight);
            }
            g2d.drawLine(i * _placeSizeWidth, 0, i * _placeSizeWidth, _pictureHeight / _placeSizeHeight * _placeSizeHeight);
        }
    }

    private void DrawObjects(Graphics g) {
        int width = _pictureWidth / _placeSizeWidth;
        int height = _pictureHeight / _placeSizeHeight;

        int i = 0;
        for (T bus : _collection.GetBuses(_collection.Count())) {
            if (bus != null) {
                // Установка позиции
                bus.SetPosition(
                        (int)((width - 1) * _placeSizeWidth - (i % width * _placeSizeWidth)),
                        (int)((height - 1) * _placeSizeHeight - (i / width * _placeSizeHeight))
                );
                // Прорисовка объекта
                bus.DrawTransport(g);
            }
            i++;
        }
    }
}
