package AccordionBus;

import java.util.Random;

// Дополнительный параметризованный набор объектов
public class SetBusGeneric<T extends EntityBus, U extends IDrawingDoors> {
    // Набор объектов от первого параметра
    private final T[] _buses;
    public int BusesCount;

    // Набор объектов от второго параметра
    private final U[] _doors;
    public int DoorsCount;

    // Конструктор
    public SetBusGeneric(int busesCount, int doorsCount) {
        _buses = (T[]) new EntityBus[busesCount];
        _doors = (U[]) new IDrawingDoors[doorsCount];
    }

    // Добавление объекта в первый набор
    public int Insert(T bus) {
        // Проверка на вместимость
        if (BusesCount >= _buses.length) {
            return -1;
        }

        _buses[BusesCount++] = bus;
        return BusesCount;
    }

    // Добавление объекта во второй набор
    public int Insert(U door) {
        // Проверка на вместимость
        if (DoorsCount >= _doors.length) {
            return -1;
        }

        _doors[DoorsCount++] = door;
        return DoorsCount;
    }

    // Создание объекта класса прорисовки
    public DrawingBus NewBus(int pictureWidth, int pictureHeight) {
        Random random = new Random();
        T bus = _buses[random.nextInt(BusesCount)];
        U door = _doors[random.nextInt(DoorsCount)];

        if (bus instanceof EntityAccordionBus) {
            return new DrawingAccordionBus(bus, door, pictureWidth, pictureHeight);
        }
        return new DrawingBus(bus, door, pictureWidth, pictureHeight);
    }
}
