package AccordionBus;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.NoSuchElementException;

// Параметризованный набор рбъектов
public class SetGeneric<T extends Object> {
    //  Список объектов, которые храним
    private final ArrayList<T> _places;

    // Количество объектов в списке
    public int Count() {
        return _places.size();
    }

    // Максимальное количество объектов в списке
    private final int _maxCount;

    // Конструктор
    public SetGeneric(int count) {
        _maxCount = count;
        _places = new ArrayList<T>(count);
    }

    // Добавление объекта в набор
    public int Insert(T bus) {
        return Insert(bus, 0);
    }

    // Добавление объекта в набор на конкретную позицию
    public int Insert(T bus, int position) {
        // Проверка позиции
        if (position < 0 || position >= _maxCount) {
            return -1;
        }

        // Вставка по позиции
        _places.add(position, bus);
        return position;
    }

    // Удаление объекта из набора с конкретной позиции
    public boolean Remove(int position) {
        // Проверка позиции
        if (position < 0 || position >= _maxCount) {
            return false;
        }

        // Удаление объекта из списка
        _places.remove(position);
        return true;
    }

    // Получение объекта из набора по позиции
    public T Get(int position) {
        // Проверка позиции
        if (position < 0 || position >= _maxCount) {
            return null;
        }

        return _places.get(position);
    }

    // Проход по списку
    public Iterable<T> GetBuses(final Integer maxBuses) {
        return new Iterable<T>() {
            @Override
            public Iterator<T> iterator() {
                return new Iterator<T>() {
                    private int currentIndex  = 0;
                    private int count = 0;

                    @Override
                    public boolean hasNext() {
                        return (currentIndex < Count()) && (maxBuses == null || count < maxBuses);
                    }

                    @Override
                    public T next() {
                        if (hasNext()) {
                            count++;
                            return _places.get(currentIndex++);
                        }
                        throw new NoSuchElementException();
                    }
                };
            }
        };
    }
}
